from fetchfox.constants.currencies import STUFF

STUFF_TOKEN_SYMBOL = STUFF

STUFF_TOKEN_POLICY_ID = "51a5e236c4de3af2b8020442e2a26f454fda3b04cb621c1294a0ef34"
STUFF_TOKEN_ASSET_NAME = "424f4f4b"

STUFF_TOKEN_ASSET_ID = f"{STUFF_TOKEN_POLICY_ID}{STUFF_TOKEN_ASSET_NAME}"

STUFF_TOKEN_COINGECKO_ID = "book-2"

STUFF_TOKEN_FINGERPRINT = "asset1lpmhzq33vsj9jpj2wsluu0usqfaf6kk96zdw5c"
